﻿--[[ Postal Express: Mouse Shortcuts for Mail ]]--
--shift-click to take item/money from mail
--ctrl-click to return mail
--alt-click to add item to SendMailFrame

assert( Postal, "Postal not found!")

------------------------------
--      Are you local?      --
------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Postal")

----------------------------------
--      Module Declaration      --
----------------------------------

Postal_Express = Postal:NewModule("Express")
Postal_Express.revision = tonumber(string.sub("$Revision: 57190 $", 12, -3))
Postal_Express.defaults = {
	EnableAltClick = true,
	AutoSend = true,
}

Postal_Express.consoleOptions = {
	type = 'group',
	name = L["Express options"],
	desc = L["Options for the Express module"],
	args = {
		EnableAltClick = {
			name = L["Enable Alt-Click to send mail"], type = 'toggle',
			desc = L["If this option is on, Alt-Clicking an item in your bags will send it instantly to the player in the To: field"],
			get = function() return Postal_Express.db.char.EnableAltClick end,
			set = function(v)
				local self = Postal_Express
				self.db.char.EnableAltClick = v
				self.consoleOptions.args.AutoSend.disabled = not v
				if v then
					if MailFrame:IsVisible() then
						self:HookScript(GameTooltip, "OnTooltipSetItem")
						self:Hook("ContainerFrameItemButton_OnModifiedClick", true)
					end
				else
					if self:IsHooked(GameTooltip, "OnTooltipSetItem") then
						self:Unhook(GameTooltip, "OnTooltipSetItem")
						self:Unhook("ContainerFrameItemButton_OnModifiedClick")
					end
				end
			end,
			order = 10,
		},
		AutoSend = {
			type = 'toggle',
			name = L["Auto-Send on Alt-Click"],
			desc = L["With this option on, Postal will mail an item as soon as it is attached if there is a recipient filled in."],
			get = function() return Postal_Express.db.char.AutoSend end,
			set = function(v) Postal_Express.db.char.AutoSend = v end,
			order = 20,
		},
	},
}

function Postal_Express:MAIL_SHOW()
	if self.db.char.EnableAltClick then
		self:HookScript(GameTooltip, "OnTooltipSetItem")
		self:Hook("ContainerFrameItemButton_OnModifiedClick", true)
	end
	self:RegisterEvent("MAIL_CLOSED", "Reset")
	self:RegisterEvent("PLAYER_LEAVING_WORLD", "Reset")
end

function Postal_Express:Reset()
	if self:IsHooked(GameTooltip, "OnTooltipSetItem") then
		self:Unhook(GameTooltip, "OnTooltipSetItem")
		self:Unhook("ContainerFrameItemButton_OnModifiedClick")
	end
	self:UnregisterEvent("MAIL_CLOSED")
	self:UnregisterEvent("PLAYER_LEAVING_WORLD")
end
	
function Postal_Express:OnEnable()
	self:Hook("InboxFrame_OnClick", true)
	self:SecureHook("InboxFrameItem_OnEnter")

	self:RegisterEvent("MAIL_SHOW")
	self.consoleOptions.args.AutoSend.disabled = not self.db.char.EnableAltClick
end

function Postal_Express:OnDisable()
	-- Disabling modules unregisters all events/hook automatically
end

-- Ctrl-Click and Shift-Clicks Shortcuts for Mail, based on ExpressMail by Tekkub
function Postal_Express:InboxFrameItem_OnEnter()
	local tooltip = GameTooltip
	local this = this
	
	local money, COD, _, hasItem, _, wasReturned, _, canReply = select(5, GetInboxHeaderInfo(this.index))
	if hasItem and hasItem > 1 then
		for i = 1, ATTACHMENTS_MAX_RECEIVE do
			local name, itemTexture, count, quality, canUse = GetInboxItem(this.index, i);
			if name then
				local itemLink = GetInboxItemLink(this.index, i);
				if count > 1 then
					tooltip:AddLine(("%sx%d"):format(itemLink, count))
				else
					tooltip:AddLine(itemLink)
				end
				tooltip:AddTexture(itemTexture)
			end
		end
	end
	if money > 0 or COD > 0 or hasItem then
		tooltip:AddLine(L["|cffeda55fShift-Click|r to take the contents."])
	end
	if not wasReturned and canReply then
		tooltip:AddLine(L["|cffeda55fCtrl-Click|r to return it to sender."])
	end
	tooltip:Show()
end

function Postal_Express:InboxFrame_OnClick(index)
	if IsShiftKeyDown() then
		self.core:PurgeMail(index, 1)
	elseif IsControlKeyDown() then
		self.core:ReturnMail(index)
	else
		self.hooks["InboxFrame_OnClick"](index)
	end
end

function Postal_Express:OnTooltipSetItem(tooltip, ...)
	local recipient = SendMailNameEditBox:GetText()
	if type(recipient) == "string" and recipient:len() > 0 and SendMailFrame:IsVisible() and not CursorHasItem() then
		tooltip:AddLine(string.format(L["|cffeda55fAlt-Click|r to send this item to %s."], recipient))
	end
	return self.hooks[tooltip].OnTooltipSetItem(tooltip, ...)
end

function Postal_Express:ContainerFrameItemButton_OnModifiedClick(...)
	if select(1, ...) == "LeftButton" and IsAltKeyDown() and SendMailFrame:IsVisible() and not CursorHasItem() then
		local bag, slot = this:GetParent():GetID(), this:GetID()
		local texture, count = GetContainerItemInfo(bag, slot)
		PickupContainerItem(bag, slot)
		ClickSendMailItemButton()
		if self.db.char.AutoSend then
			for i = 1, ATTACHMENTS_MAX_SEND do
				-- get info about the attachment
				local itemName, itemTexture, stackCount, quality = GetSendMailItem(i);
				if SendMailNameEditBox:GetText() ~= "" and texture == itemTexture and count == stackCount then
					SendMailFrame_SendMail()
				end
			end
		end
	else
		self.hooks["ContainerFrameItemButton_OnModifiedClick"](...)
	end
end
